import * as __WEBPACK_EXTERNAL_MODULE_process__ from "process";
import * as __WEBPACK_EXTERNAL_MODULE_googleapis__ from "googleapis";
import * as __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__ from "ts-regex-builder";
import * as __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__ from "@callstack/byorg-utils";
import * as __WEBPACK_EXTERNAL_MODULE_pdfjs_dist__ from "pdfjs-dist";
import * as __WEBPACK_EXTERNAL_MODULE_lodash__ from "lodash";
import * as __WEBPACK_EXTERNAL_MODULE_fs_promises__ from "fs/promises";
import * as __WEBPACK_EXTERNAL_MODULE_path__ from "path";
import * as __WEBPACK_EXTERNAL_MODULE__google_cloud_local_auth__ from "@google-cloud/local-auth";
import * as __WEBPACK_EXTERNAL_MODULE__notionhq_client__ from "@notionhq/client";
import * as __WEBPACK_EXTERNAL_MODULE_markdown_table__ from "markdown-table";
import * as __WEBPACK_EXTERNAL_MODULE_node_path__ from "node:path";
function boundingBoxesOverlap(box1, box2) {
    return box2.x <= box1.x + box1.width && box2.x + box2.width >= box1.x && box2.y <= box1.y + box1.height && box2.y + box2.height >= box1.y;
}
function expandBoundingBox(box1, box2) {
    const xMin = Math.min(box1.x, box2.x);
    const yMin = Math.min(box1.y, box2.y);
    const xMax = Math.max(box1.x + box1.width, box2.x + box2.width);
    const yMax = Math.max(box1.y + box1.height, box2.y + box2.height);
    const newWidth = xMax - xMin;
    const newHeight = yMax - yMin;
    return {
        x: xMin,
        y: yMin,
        width: newWidth,
        height: newHeight
    };
}
async function getLinkBoundingBoxes(page) {
    const annotations = await page.getAnnotations({
        intent: 'display'
    });
    const bboxes = annotations.filter((annot)=>'Link' === annot.subtype && annot.rect && annot.url).map((annot)=>({
            url: annot.url,
            bbox: {
                x: annot.rect[0],
                y: annot.rect[1],
                width: annot.rect[2] - annot.rect[0],
                height: annot.rect[3] - annot.rect[1]
            }
        }));
    // Remove duplicates
    return (0, __WEBPACK_EXTERNAL_MODULE_lodash__.uniqWith)(bboxes, (a, b)=>boundingBoxesOverlap(a.bbox, b.bbox) && a.url === b.url);
}
async function loadPdfContentFromBuffer(buffer) {
    const document = await __WEBPACK_EXTERNAL_MODULE_pdfjs_dist__.getDocument(buffer).promise;
    let text = '';
    // pdf.js counts pages from 1 to N
    for(let pageNum = 1; pageNum <= document.numPages; pageNum++){
        const page = await document.getPage(pageNum);
        const textContent = await page.getTextContent();
        const links = await getLinkBoundingBoxes(page);
        const textItems = normalize(textContent);
        const textBlocks = joinAdjacentPdfTexts(textItems);
        // The top-left corner is at (0, N), where N > 0.
        // Therefore, we need to sort Y in descending order and X in ascending order.
        const orderedTextBlocks = (0, __WEBPACK_EXTERNAL_MODULE_lodash__.orderBy)(textBlocks, [
            'y',
            'x'
        ], [
            'desc',
            'asc'
        ]);
        applyLinks(orderedTextBlocks, links);
        text += orderedTextBlocks.map((block)=>block.text).join('\n');
        text += `-Page (${pageNum}) Break-\n`;
    }
    return text;
}
// EOL -> end of line
function normalize(textContent) {
    return textContent.items.filter((item)=>!('id' in item)).map((textItem)=>({
            text: textItem.str,
            x: textItem.transform[4],
            y: textItem.transform[5],
            width: textItem.width,
            height: textItem.height,
            hasEOL: textItem.hasEOL
        }));
}
/**
 * Combines text lines into cohesive multi-line text blocks.
 */ function joinAdjacentPdfTexts(texts) {
    const joinedTexts = [];
    let currentTextItem = null;
    for (const textItem of texts){
        if (currentTextItem) {
            // Empty texts appear in odd locations; we need to ignore them while expanding the bounding box.
            if ('' !== textItem.text) currentTextItem = {
                ...currentTextItem,
                ...expandBoundingBox(currentTextItem, textItem)
            };
            currentTextItem.text += textItem.text;
        } else currentTextItem = textItem;
        // The following rules were identified during the analysis of the pdf.js output. They might not be accurate in all cases.
        if (textItem.hasEOL) {
            if ('' === textItem.text) {
                // No text, but hasEOL -> should start a new block
                joinedTexts.push(currentTextItem);
                currentTextItem = null;
            } else // Text with EOL -> should add space and continue building
            currentTextItem.text += ' ';
        }
    }
    // Edge case: last element on the page without an EOL.
    if (currentTextItem) joinedTexts.push(currentTextItem);
    return joinedTexts;
}
function applyLinks(textBlocks, links) {
    // In PDFs, links are annotations that hover over the document.
    // We need to repeatedly iterate over all the links to find those overlapping specific text blocks.
    textBlocks.forEach((block)=>{
        links.filter((link)=>boundingBoxesOverlap(link.bbox, block)).forEach((link)=>{
            block.text += ` [link](${link.url})`;
        });
    });
}
const contentWhitespace = (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.anyOf)(' \t\x0b');
const contentWhitespacesRegex = (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)((0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(contentWhitespace), {
    global: true
});
const leadingWhitespaceRegex = (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.lookbehind)((0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)('\n')),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.whitespace)
], {
    global: true
});
const trailingWhitespaceRegex = (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.whitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.lookahead)((0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)('\n'))
], {
    global: true
});
// Callstack  |   IS.11 - Asset Register Policy Confidentiality level: 3 - Confidential
const deniedConfidentialyLevelRegex = (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    'Confidentiality',
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    'level:',
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    '3',
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('-'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    'Confidential'
], {
    global: true,
    ignoreCase: true
});
// Callstack  |   IS.11 - Asset Register Policy Confidentiality level: 2 - Internal
// Callstack | IS.11 - Asset Register
const allowedConfidentialyLevelRegex = (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('Callstack'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('|'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('IS.11'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('-'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('Asset'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('Register'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('Policy'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    'Confidentiality',
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    'level:',
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.choiceOf)('1', '2'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('-'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(contentWhitespace),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.choiceOf)('Public', 'Internal')
], {
    global: true,
    ignoreCase: true
});
const pageBreakRegex = (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)('-'),
    'Page (',
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.digit),
    ') Break',
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)('-')
], {
    global: true
});
const lineSeparatorRegex = (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(' '),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.choiceOf)([
        (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)('\n'),
        (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('\n')
    ], [
        (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)('\r'),
        (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)('\n')
    ]),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.zeroOrMore)(' ')
], {
    global: true
});
// Markdown download escapes all dashes \-
(0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    '\\-'
], {
    global: true
});
// Reformat all bullet lists
(0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)((0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.choiceOf)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.startOfString, '\n')),
    '●',
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.whitespace)
], {
    global: true
});
/*
![][image1]

[image1]:
<data:image/png;base64,iVBORw0K>
*/ (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.choiceOf)([
        '![][',
        (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.any, {
            greedy: false
        }),
        ']'
    ], [
        '[',
        (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.any, {
            greedy: false
        }),
        ']:',
        (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.optional)((0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.choiceOf)((0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)('\n'), (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.whitespace))),
        '<',
        (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.any, {
            greedy: false
        }),
        '>'
    ])
], {
    global: true
});
(0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    '[',
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.any, {
        greedy: false
    }),
    ']',
    '(',
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.any, {
        greedy: false
    }),
    ')'
], {
    global: true
});
function parsePdfContentToPages(content) {
    const pages = content.split(pageBreakRegex);
    return pages.map(normalizePageContent).filter((page)=>page);
}
function normalizePageContent(content) {
    if (deniedConfidentialyLevelRegex.test(content)) throw new Error('PDF Parser Confidentiality level is not allowed');
    return content.replaceAll(lineSeparatorRegex, '\n').replaceAll(leadingWhitespaceRegex, '').replaceAll(trailingWhitespaceRegex, '').replaceAll(contentWhitespacesRegex, ' ').replaceAll(allowedConfidentialyLevelRegex, '').trim();
}
// Identifies file id in a url from google drive
const fileIdChar = (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.charClass)(__WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.word, (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.anyOf)('-'));
const fileIdRegex = (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.buildRegExp)([
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.lookbehind)('/d/'),
    (0, __WEBPACK_EXTERNAL_MODULE_ts_regex_builder__.oneOrMore)(fileIdChar)
]);
function extractFileIdFromUrl(fileUrl) {
    const url = new URL(fileUrl);
    const fileId = url.pathname.match(fileIdRegex)?.[0];
    if (!fileId) {
        __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.logger.error("Didn't recognize file Id");
        __WEBPACK_EXTERNAL_MODULE_process__["default"].exit(0);
    }
    return fileId;
}
async function fetchFileMetadata(authClient, url) {
    const fileId = extractFileIdFromUrl(url);
    const drive = __WEBPACK_EXTERNAL_MODULE_googleapis__.google.drive({
        version: 'v3',
        auth: authClient
    });
    const response = await drive.files.get({
        fileId,
        fields: 'name,mimeType,modifiedTime,labelInfo'
    });
    const metadata = response?.data;
    if (!metadata?.name) return null;
    return {
        url,
        title: metadata.name,
        lastModified: metadata.modifiedTime ? new Date(metadata.modifiedTime) : void 0
    };
}
async function fetchPdfFileAsPages(authClient, fileUrl) {
    const fileId = extractFileIdFromUrl(fileUrl);
    const drive = __WEBPACK_EXTERNAL_MODULE_googleapis__.google.drive({
        version: 'v3',
        auth: authClient
    });
    const response = await drive.files.get({
        fileId,
        alt: 'media'
    }, {
        responseType: 'arraybuffer'
    });
    const buffer = await response.data;
    const content = await loadPdfContentFromBuffer(buffer);
    return parsePdfContentToPages(content);
}
async function fetchSlidesAsPages(authClient, url) {
    const slidesRootUrl = 'https://docs.google.com/presentation/d/';
    const fileId = extractFileIdFromUrl(url);
    const formattedUrl = `${slidesRootUrl}${fileId}/export/pdf?id=${fileId}`;
    const headers = await authClient.getRequestHeaders();
    const response = await fetch(formattedUrl, {
        headers,
        redirect: 'follow',
        credentials: 'include'
    });
    if (200 !== response.status) throw new Error(`GoogleDrive/fetchSlidesAsPages Invalid response status: ${response.status}`);
    const buffer = await response.arrayBuffer();
    const content = await loadPdfContentFromBuffer(buffer);
    return parsePdfContentToPages(content);
}
async function fetchDocAsMarkdown(authClient, fileUrl) {
    const fileId = extractFileIdFromUrl(fileUrl);
    const drive = __WEBPACK_EXTERNAL_MODULE_googleapis__.google.drive({
        version: 'v3',
        auth: authClient
    });
    const response = await drive.files.export({
        fileId,
        alt: 'media',
        mimeType: 'text/markdown'
    }, {
        responseType: 'arraybuffer'
    });
    if (200 !== response.status) throw new Error(`GoogleDrive/fetchDocAsMarkdown Invalid response status: ${response.status}`);
    // @ts-expect-error googleapis typic is incorrect
    const buffer = Buffer.from(response.data);
    return new TextDecoder().decode(buffer);
}
// If modifying these scopes, delete token.json.
const SCOPES = [
    'https://www.googleapis.com/auth/drive.readonly'
];
// The file token.json stores the user's access and refresh tokens, and is
// created automatically when the authorization flow completes for the first
// time.
const TOKEN_PATH = __WEBPACK_EXTERNAL_MODULE_path__["default"].join(__WEBPACK_EXTERNAL_MODULE_process__["default"].cwd(), 'google-token.json');
const CREDENTIALS_PATH = __WEBPACK_EXTERNAL_MODULE_path__["default"].join(__WEBPACK_EXTERNAL_MODULE_process__["default"].cwd(), 'google-credentials.json');
async function loadClientFromCredentialsIfExist() {
    try {
        const content = await __WEBPACK_EXTERNAL_MODULE_fs_promises__["default"].readFile(TOKEN_PATH);
        const credentials = JSON.parse(content.toString());
        __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.logger.debug('Google: authorized with token');
        return __WEBPACK_EXTERNAL_MODULE_googleapis__.google.auth.fromJSON(credentials);
    } catch (err) {
        throw new Error(`GoogleAuth Authorization error: ${err}`);
    }
}
async function saveCredentials(client) {
    const content = await __WEBPACK_EXTERNAL_MODULE_fs_promises__["default"].readFile(CREDENTIALS_PATH);
    const keys = JSON.parse(content.toString());
    const key = keys.installed || keys.web;
    const payload = JSON.stringify({
        type: 'authorized_user',
        client_id: key.client_id,
        client_secret: key.client_secret,
        refresh_token: client.credentials.refresh_token
    });
    await __WEBPACK_EXTERNAL_MODULE_fs_promises__["default"].writeFile(TOKEN_PATH, payload);
}
let auth_authClient = null;
async function getGoogleAuthClient() {
    if (auth_authClient) return auth_authClient;
    auth_authClient = await authorize();
    return auth_authClient;
}
async function authorize() {
    let client = await loadClientFromCredentialsIfExist();
    if (client) return client;
    client = await (0, __WEBPACK_EXTERNAL_MODULE__google_cloud_local_auth__.authenticate)({
        scopes: SCOPES,
        keyfilePath: CREDENTIALS_PATH
    });
    if (client?.credentials) await saveCredentials(client);
    return client;
}
const URL_ROOT = 'https://drive.google.com/file/d/';
const googleDriveLoader = {
    isSupported,
    loadDocument
};
function isSupported(source) {
    return source.startsWith(URL_ROOT);
}
async function loadDocument(url) {
    const authClient = await getGoogleAuthClient();
    const fileMetadata = await fetchFileMetadata(authClient, url);
    if (!fileMetadata) throw new Error(`DocumentLoader/GoogleDrive Document name not found: ${url}`);
    const contentChunks = await fetchPdfFileAsPages(authClient, url);
    return {
        content: contentChunks.join('\n\n'),
        metadata: {
            ...fileMetadata,
            url: formatStandardizedUrl(url)
        }
    };
}
function formatStandardizedUrl(url) {
    const fileId = extractFileIdFromUrl(url);
    return `${URL_ROOT}${fileId}`;
}
const google_slides_URL_ROOT = 'https://docs.google.com/presentation/d/';
const googleSlidesLoader = {
    isSupported: google_slides_isSupported,
    loadDocument: google_slides_loadDocument
};
function google_slides_isSupported(source) {
    return source.startsWith(google_slides_URL_ROOT);
}
async function loadMetadata(url) {
    const authClient = await getGoogleAuthClient();
    const fileMetadata = await fetchFileMetadata(authClient, url);
    if (!fileMetadata) throw new Error(`DocumentLoader/GoogleSlides Document name not found: ${url}`);
    return {
        ...fileMetadata,
        url: google_slides_formatStandardizedUrl(url)
    };
}
async function google_slides_loadDocument(url) {
    const metadata = await loadMetadata(url);
    const authClient = await getGoogleAuthClient();
    const slides = await fetchSlidesAsPages(authClient, url);
    return {
        content: slides.join('\n\n'),
        metadata
    };
}
function google_slides_formatStandardizedUrl(url) {
    const fileId = extractFileIdFromUrl(url);
    return `${google_slides_URL_ROOT}${fileId}`;
}
const google_docs_URL_ROOT = 'https://docs.google.com/document/d/';
const googleDocsLoader = {
    isSupported: google_docs_isSupported,
    loadDocument: google_docs_loadDocument
};
function google_docs_isSupported(source) {
    return source.startsWith(google_docs_URL_ROOT);
}
async function google_docs_loadMetadata(url) {
    const authClient = await getGoogleAuthClient();
    const fileMetadata = await fetchFileMetadata(authClient, url);
    if (!fileMetadata) throw new Error(`DocumentLoader/GoogleDocs Document metadata not found: ${url}`);
    return {
        ...fileMetadata,
        url: google_docs_formatStandardizedUrl(url)
    };
}
async function google_docs_loadDocument(url) {
    const metadata = await google_docs_loadMetadata(url);
    const authClient = await getGoogleAuthClient();
    const content = await fetchDocAsMarkdown(authClient, url);
    return {
        content,
        metadata
    };
}
function google_docs_formatStandardizedUrl(url) {
    const fileId = extractFileIdFromUrl(url);
    return `${google_docs_URL_ROOT}${fileId}`;
}
function heading(text, level = 1) {
    return `${'#'.repeat(level)} ${text}`;
}
function bold(text) {
    return `**${text}**`;
}
function italic(text) {
    return `*${text}*`;
}
function strikethrough(text) {
    return `~~${text}~~`;
}
function codeBlock(text, lang = '') {
    return '```' + lang + '\n' + text + '\n```\n';
}
function orderedListItem(text) {
    return `1. ${text}`;
}
function unorderedListItem(text) {
    return `- ${text}`;
}
function toDoItem(text, checked) {
    return `- [${checked ? 'x' : ' '}] ${text}`;
}
function builder_link(text, href) {
    return `[${text}](${href})`;
}
function blockQuote(text) {
    const lines = text.split('\n');
    return lines.map((line)=>`> ${line}`).join('\n');
}
const divider = '\n---\n';
function builder_table(table) {
    return (0, __WEBPACK_EXTERNAL_MODULE_markdown_table__["default"])(table, {
        alignDelimiters: false
    });
}
function parseOptionalStringProp(props, name) {
    const prop = props[name];
    if (null == prop) return;
    switch(prop.type){
        case 'rich_text':
            if (0 === prop.rich_text.length) return;
            return prop.rich_text.map((t)=>t.plain_text).join('');
        case 'title':
            return prop.title[0]?.plain_text ?? void 0;
        case 'url':
            return prop.url ?? void 0;
        case 'email':
            return prop.email ?? void 0;
        case 'select':
            return prop.select?.name;
        case 'multi_select':
            return prop.multi_select.map((s)=>s.name).join('; ');
        default:
            __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.logger.error(`${name}:`, JSON.stringify(prop, null, 2));
            throw new Error(`Notion: property type "${prop?.type}" not supported ("${name}")`);
    }
}
let notionClient;
function auth_getNotionClient() {
    if (notionClient) return notionClient;
    const notionToken = (0, __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.requireEnv)('NOTION_TOKEN');
    return new __WEBPACK_EXTERNAL_MODULE__notionhq_client__.Client({
        auth: notionToken
    });
}
async function fetchPageMetadata(pageId) {
    try {
        const notion = auth_getNotionClient();
        const page = await notion.pages.retrieve({
            page_id: pageId
        });
        if ('page' !== page.object || !('properties' in page)) throw new Error('Notion invalid page object');
        const title = parseOptionalStringProp(page.properties, 'Page') ?? parseOptionalStringProp(page.properties, 'title');
        if (null == title) throw new Error('Notion page title not found');
        return {
            url: page.url,
            title: title,
            lastModified: new Date(page.last_edited_time)
        };
    } catch (error) {
        throw Error(`Notion failed to fetch page metadata ${error}`);
    }
}
async function fetchBlock(blockId) {
    __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.logger.debug(`Notion: fetching ${blockId} block...`);
    const notion = auth_getNotionClient();
    const result = await notion.blocks.retrieve({
        block_id: blockId
    });
    __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.logger.debug(`Notion: fetched ${blockId} block`);
    return result;
}
async function fetchChildBlocks(blockId) {
    __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.logger.debug(`Notion: fetching ${blockId} child block...`);
    const notion = auth_getNotionClient();
    const result = await (0, __WEBPACK_EXTERNAL_MODULE__notionhq_client__.collectPaginatedAPI)(notion.blocks.children.list, {
        block_id: blockId
    });
    __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.logger.debug(`Notion: fetched ${result.length} child block`);
    return result;
}
async function parseBlocks(blocks, context) {
    const result = [];
    for (const block of blocks){
        const parsed = await parseBlock(block, context);
        if (parsed) result.push(parsed);
    }
    return result.join('\n');
}
async function parseBlock(block, context) {
    if (!(0, __WEBPACK_EXTERNAL_MODULE__notionhq_client__.isFullBlock)(block)) return null;
    let markdown = '';
    let parseChildren = true;
    switch(block.type){
        case 'bulleted_list_item':
            markdown = unorderedListItem(parseRichText(block.bulleted_list_item.rich_text));
            break;
        case 'bookmark':
            markdown = builder_link(parseRichText(block.bookmark.caption), block.bookmark.url);
            break;
        case 'callout':
            markdown = parseRichText(block.callout.rich_text);
            break;
        case 'code':
            markdown = parseCodeBlock(block);
            break;
        case 'heading_1':
            markdown = heading(parseRichText(block.heading_1.rich_text), 1);
            break;
        case 'heading_2':
            markdown = heading(parseRichText(block.heading_2.rich_text), 2);
            break;
        case 'heading_3':
            markdown = heading(parseRichText(block.heading_3.rich_text), 3);
            break;
        case 'link_to_page':
            markdown = await parseLinkToPageBlock(block);
            break;
        case 'numbered_list_item':
            markdown = orderedListItem(parseRichText(block.numbered_list_item.rich_text));
            break;
        case 'paragraph':
            markdown = `${parseRichText(block.paragraph.rich_text)}\n`;
            break;
        case 'quote':
            markdown = blockQuote(parseRichText(block.quote.rich_text));
            break;
        case 'template':
            markdown = parseRichText(block.template.rich_text);
            break;
        case 'to_do':
            markdown = toDoItem(parseRichText(block.to_do.rich_text), block.to_do.checked);
            break;
        case 'toggle':
            markdown = parseRichText(block.toggle.rich_text);
            break;
        case 'divider':
            markdown = divider;
            parseChildren = false;
            break;
        case 'table':
            markdown = await parseTableBlocks(block);
            parseChildren = false;
            break;
        case 'child_page':
            markdown = await parseChildPageBlock(block);
            parseChildren = false;
            break;
        case 'pdf':
            {
                const blockUrl = new URL(context.pageUrl);
                blockUrl.hash = block.id;
                __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.logger.warn(`Ignoring embedded PDF: ${blockUrl.toString()}`);
                return null;
            }
        // Supported blocks that do not have content but will have their children parsed:
        case 'column_list':
        case 'column':
            break;
        // Skipped blocks, the children will nnot be parsed:
        case 'child_database':
        case 'image':
        case 'video':
        case 'audio':
        case 'file':
        case 'unsupported':
            parseChildren = false;
            break;
        default:
            parseChildren = false;
            __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.logger.debug('Notion: block type not implemented:', block.type);
            break;
    }
    markdown = indentText(markdown, context.depth);
    if (parseChildren && block.has_children) {
        markdown += '\n';
        // 'column_list' and 'column' shouldn't influence depth as those are completely stripped from Markdown
        const nextDepth = [
            'column_list',
            'column'
        ].includes(block.type) ? context.depth : context.depth + 1;
        const children = await fetchChildBlocks(block.id);
        markdown += await parseBlocks(children, {
            ...context,
            depth: nextDepth
        });
    }
    return markdown;
}
function parseRichText(items) {
    let result = [];
    items.forEach((item)=>{
        let text = item.plain_text;
        if (item.href) text = builder_link(text, item.href);
        if (item.annotations.bold) text = bold(text);
        if (item.annotations.italic) text = italic(text);
        if (item.annotations.strikethrough) text = strikethrough(text);
        result.push(text);
    });
    return result.join('').trim();
}
async function parseLinkToPageBlock(block) {
    if ('page_id' !== block.link_to_page.type) {
        __WEBPACK_EXTERNAL_MODULE__callstack_byorg_utils__.logger.error('Notion: block link_to_page.type not supported:', block.link_to_page.type, JSON.stringify(block.link_to_page, null, 2));
        throw new Error(`Notion block link_to_page.type "${block.link_to_page.type}" not supported.`);
    }
    const metadata = await fetchPageMetadata(block.link_to_page.page_id);
    return builder_link(metadata.title, metadata.url);
}
async function parseChildPageBlock(block) {
    const metadata = await fetchPageMetadata(block.id);
    return builder_link(metadata.title, metadata.url);
}
function parseCodeBlock(block) {
    return codeBlock(parseRichText(block.code.rich_text), 'plain text' === block.code.language ? 'text' : block.code.language);
}
async function parseTableBlocks(tableBlock) {
    const rows = [];
    if (!tableBlock.table.has_column_header) rows.push(Array(tableBlock.table.table_width).fill(''));
    const children = await fetchChildBlocks(tableBlock.id);
    for (const child of children)if ((0, __WEBPACK_EXTERNAL_MODULE__notionhq_client__.isFullBlock)(child) && 'table_row' === child.type) rows.push(child.table_row.cells.map((cell)=>excapeNewlines(parseRichText(cell))));
    return builder_table(rows);
}
function excapeNewlines(text) {
    return text.replace(/\n/g, '<br/>');
}
const INDENT_STEP = '  ';
function indentText(text, n) {
    return text.split('\n').map((line)=>`${INDENT_STEP.repeat(n)}${line}`).join('\n');
}
const notion_URL_ROOT = 'https://www.notion.so/';
const notionLoader = {
    isSupported: notion_isSupported,
    loadDocument: notion_loadDocument
};
function notion_isSupported(source) {
    return source.startsWith(notion_URL_ROOT);
}
// TODO: implement document loading
async function notion_loadDocument(url) {
    const pageId = extractNotionPageId(url);
    if (!pageId) throw new Error(`NotionLoader Invalid Notion URL: ${url}`);
    const metadata = await fetchPageMetadata(pageId);
    const blocks = await fetchChildBlocks(pageId);
    const content = await parseBlocks(blocks, {
        pageUrl: url,
        depth: 0
    });
    return {
        content,
        metadata
    };
}
function extractNotionPageId(url) {
    const pattern = /[0-9a-f]{32}/;
    const match = url.match(pattern);
    return match ? match[0] : null;
}
function extractFilename(url) {
    return __WEBPACK_EXTERNAL_MODULE_node_path__["default"].basename(new URL(url).pathname);
}
const notion_pdf_block_URL_ROOT = 'https://www.notion.so/';
const BLOCK_ID_REGEX = /^[a-z0-9]{32}$/;
// 32 characters
const notionPdfBlockLoader = {
    isSupported: notion_pdf_block_isSupported,
    loadDocument: notion_pdf_block_loadDocument
};
function notion_pdf_block_isSupported(source) {
    return source.startsWith(notion_pdf_block_URL_ROOT) && BLOCK_ID_REGEX.test(getUrlHash(source));
}
async function notion_pdf_block_loadDocument(url) {
    const pdfBlock = await fetchPdfBlock(url);
    const metadata = getMetadataFromBlock(url, pdfBlock);
    const pdfUrl = getPdfUrlFromBlock(pdfBlock);
    const response = await fetch(pdfUrl);
    const buffer = await response.arrayBuffer();
    const content = await loadPdfContentFromBuffer(buffer);
    return {
        content,
        metadata
    };
}
function getUrlHash(url) {
    const urlHash = new URL(url).hash;
    // Remove '#' symbol
    return urlHash.slice(1);
}
async function fetchPdfBlock(url) {
    const blockId = getUrlHash(url);
    const block = await fetchBlock(blockId);
    if (!block) throw new Error(`NotionPdfBlockLoader Notion block not found: ${blockId}`);
    if (!(0, __WEBPACK_EXTERNAL_MODULE__notionhq_client__.isFullBlock)(block)) throw new Error('NotionPdfBlockLoader Invalid block object');
    if ('pdf' !== block.type) throw new Error('NotionPdfBlockLoader Given block is not an embedded PDF');
    return block;
}
function getMetadataFromBlock(url, pdfBlock) {
    const pdfUrl = getPdfUrlFromBlock(pdfBlock);
    return {
        url,
        title: extractFilename(pdfUrl),
        lastModified: new Date(pdfBlock.last_edited_time)
    };
}
function getPdfUrlFromBlock(block) {
    if ('external' === block.pdf.type) return block.pdf.external.url;
    return block.pdf.file.url;
}
const loaders = [
    notionPdfBlockLoader,
    googleDriveLoader,
    googleSlidesLoader,
    googleDocsLoader,
    notionLoader
];
function loaders_loadDocument(source) {
    for (const loader of loaders)if (loader.isSupported(source)) return loader.loadDocument(source);
    throw new Error(`DocumentLoader Unsupported document type ${source}`);
}
export { googleDocsLoader, googleDriveLoader, googleSlidesLoader, loaders_loadDocument as loadDocument, notionLoader, notionPdfBlockLoader };
